from dataclasses import dataclass
from typing import Optional

from frostfs_testlib.storage.dataclasses.wallet import WalletInfo
from frostfs_testlib.testing.readable import HumanReadableEnum


@dataclass
class ObjectRef:
    cid: str
    oid: str


@dataclass
class LockObjectInfo(ObjectRef):
    lifetime: Optional[int] = None
    expire_at: Optional[int] = None


@dataclass
class StorageObjectInfo(ObjectRef):
    size: Optional[int] = None
    wallet: Optional[WalletInfo] = None
    file_path: Optional[str] = None
    file_hash: Optional[str] = None
    attributes: Optional[list[dict[str, str]]] = None
    tombstone: Optional[str] = None
    locks: Optional[list[LockObjectInfo]] = None


class NodeStatus(HumanReadableEnum):
    MAINTENANCE: str = "maintenance"
    ONLINE: str = "online"
    OFFLINE: str = "offline"


@dataclass
class NodeNetmapInfo:
    node_id: str = None
    node_status: NodeStatus = None
    node_data_ips: list[str] = None
    cluster_name: str = None
    continent: str = None
    country: str = None
    country_code: str = None
    external_address: list[str] = None
    location: str = None
    node: str = None
    price: int = None
    sub_div: str = None
    sub_div_code: int = None
    un_locode: str = None
    role: str = None


class Interfaces(HumanReadableEnum):
    DATA_O: str = "data0"
    DATA_1: str = "data1"
    MGMT: str = "mgmt"
    INTERNAL_0: str = "internal0"
    INTERNAL_1: str = "internal1"


@dataclass
class NodeNetInfo:
    epoch: str = None
    network_magic: str = None
    time_per_block: str = None
    container_fee: str = None
    epoch_duration: str = None
    inner_ring_candidate_fee: str = None
    maximum_object_size: str = None
    withdrawal_fee: str = None
    homomorphic_hashing_disabled: str = None
    maintenance_mode_allowed: str = None
    eigen_trust_alpha: str = None
    eigen_trust_iterations: str = None
